# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .batch_eligibility_response import BatchEligibilityResponse
    from .coordination_of_benefits_recommendation import CoordinationOfBenefitsRecommendation
    from .copay_estimation_recommendation import CopayEstimationRecommendation
    from .copay_estimation_recommendation_payload import CopayEstimationRecommendationPayload
    from .eligibility_check import EligibilityCheck
    from .eligibility_check_error import EligibilityCheckError
    from .eligibility_check_error_details import EligibilityCheckErrorDetails
    from .eligibility_check_error_source import EligibilityCheckErrorSource
    from .eligibility_check_metadata import EligibilityCheckMetadata
    from .eligibility_check_page import EligibilityCheckPage
    from .eligibility_check_status import EligibilityCheckStatus
    from .eligibility_recommendation import EligibilityRecommendation
    from .eligibility_recommendation_patient_info import EligibilityRecommendationPatientInfo
    from .eligibility_recommendation_payload import (
        EligibilityRecommendationPayload,
        EligibilityRecommendationPayload_CoordinationOfBenefits,
        EligibilityRecommendationPayload_CopayEstimation,
        EligibilityRecommendationPayload_MedicareAdvantage,
        EligibilityRecommendationPayload_UserConfiguredPrompts,
    )
    from .eligibility_request import EligibilityRequest
    from .eligibility_response import EligibilityResponse
    from .eligibility_status import EligibilityStatus
    from .encounter import Encounter
    from .individual_provider import IndividualProvider
    from .medicare_advantage_recommendation import MedicareAdvantageRecommendation
    from .medicare_advantage_recommendation_payload import MedicareAdvantageRecommendationPayload
    from .organization_provider import OrganizationProvider
    from .parsed_response import ParsedResponse
    from .payer_search_response import PayerSearchResponse
    from .post_eligibility_recommendation_request import PostEligibilityRecommendationRequest
    from .provider import Provider
    from .request_correction import RequestCorrection
    from .stedi_payer import StediPayer
    from .stedi_payer_item import StediPayerItem
    from .user_configured_prompts_recommendation import UserConfiguredPromptsRecommendation
    from .user_configured_prompts_recommendation_payload import UserConfiguredPromptsRecommendationPayload
    from .user_configured_prompts_result import UserConfiguredPromptsResult
    from .vote import Vote
    from .vote_value import VoteValue
_dynamic_imports: typing.Dict[str, str] = {
    "BatchEligibilityResponse": ".batch_eligibility_response",
    "CoordinationOfBenefitsRecommendation": ".coordination_of_benefits_recommendation",
    "CopayEstimationRecommendation": ".copay_estimation_recommendation",
    "CopayEstimationRecommendationPayload": ".copay_estimation_recommendation_payload",
    "EligibilityCheck": ".eligibility_check",
    "EligibilityCheckError": ".eligibility_check_error",
    "EligibilityCheckErrorDetails": ".eligibility_check_error_details",
    "EligibilityCheckErrorSource": ".eligibility_check_error_source",
    "EligibilityCheckMetadata": ".eligibility_check_metadata",
    "EligibilityCheckPage": ".eligibility_check_page",
    "EligibilityCheckStatus": ".eligibility_check_status",
    "EligibilityRecommendation": ".eligibility_recommendation",
    "EligibilityRecommendationPatientInfo": ".eligibility_recommendation_patient_info",
    "EligibilityRecommendationPayload": ".eligibility_recommendation_payload",
    "EligibilityRecommendationPayload_CoordinationOfBenefits": ".eligibility_recommendation_payload",
    "EligibilityRecommendationPayload_CopayEstimation": ".eligibility_recommendation_payload",
    "EligibilityRecommendationPayload_MedicareAdvantage": ".eligibility_recommendation_payload",
    "EligibilityRecommendationPayload_UserConfiguredPrompts": ".eligibility_recommendation_payload",
    "EligibilityRequest": ".eligibility_request",
    "EligibilityResponse": ".eligibility_response",
    "EligibilityStatus": ".eligibility_status",
    "Encounter": ".encounter",
    "IndividualProvider": ".individual_provider",
    "MedicareAdvantageRecommendation": ".medicare_advantage_recommendation",
    "MedicareAdvantageRecommendationPayload": ".medicare_advantage_recommendation_payload",
    "OrganizationProvider": ".organization_provider",
    "ParsedResponse": ".parsed_response",
    "PayerSearchResponse": ".payer_search_response",
    "PostEligibilityRecommendationRequest": ".post_eligibility_recommendation_request",
    "Provider": ".provider",
    "RequestCorrection": ".request_correction",
    "StediPayer": ".stedi_payer",
    "StediPayerItem": ".stedi_payer_item",
    "UserConfiguredPromptsRecommendation": ".user_configured_prompts_recommendation",
    "UserConfiguredPromptsRecommendationPayload": ".user_configured_prompts_recommendation_payload",
    "UserConfiguredPromptsResult": ".user_configured_prompts_result",
    "Vote": ".vote",
    "VoteValue": ".vote_value",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BatchEligibilityResponse",
    "CoordinationOfBenefitsRecommendation",
    "CopayEstimationRecommendation",
    "CopayEstimationRecommendationPayload",
    "EligibilityCheck",
    "EligibilityCheckError",
    "EligibilityCheckErrorDetails",
    "EligibilityCheckErrorSource",
    "EligibilityCheckMetadata",
    "EligibilityCheckPage",
    "EligibilityCheckStatus",
    "EligibilityRecommendation",
    "EligibilityRecommendationPatientInfo",
    "EligibilityRecommendationPayload",
    "EligibilityRecommendationPayload_CoordinationOfBenefits",
    "EligibilityRecommendationPayload_CopayEstimation",
    "EligibilityRecommendationPayload_MedicareAdvantage",
    "EligibilityRecommendationPayload_UserConfiguredPrompts",
    "EligibilityRequest",
    "EligibilityResponse",
    "EligibilityStatus",
    "Encounter",
    "IndividualProvider",
    "MedicareAdvantageRecommendation",
    "MedicareAdvantageRecommendationPayload",
    "OrganizationProvider",
    "ParsedResponse",
    "PayerSearchResponse",
    "PostEligibilityRecommendationRequest",
    "Provider",
    "RequestCorrection",
    "StediPayer",
    "StediPayerItem",
    "UserConfiguredPromptsRecommendation",
    "UserConfiguredPromptsRecommendationPayload",
    "UserConfiguredPromptsResult",
    "Vote",
    "VoteValue",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawCoveragesClient, RawCoveragesClient

if typing.TYPE_CHECKING:
    from .resources.v_1.client import AsyncV1Client, V1Client


class CoveragesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCoveragesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._v_1: typing.Optional[V1Client] = None

    @property
    def with_raw_response(self) -> RawCoveragesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCoveragesClient
        """
        return self._raw_client

    @property
    def v_1(self):
        if self._v_1 is None:
            from .resources.v_1.client import V1Client  # noqa: E402

            self._v_1 = V1Client(client_wrapper=self._client_wrapper)
        return self._v_1


class AsyncCoveragesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCoveragesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._v_1: typing.Optional[AsyncV1Client] = None

    @property
    def with_raw_response(self) -> AsyncRawCoveragesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCoveragesClient
        """
        return self._raw_client

    @property
    def v_1(self):
        if self._v_1 is None:
            from .resources.v_1.client import AsyncV1Client  # noqa: E402

            self._v_1 = AsyncV1Client(client_wrapper=self._client_wrapper)
        return self._v_1



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .external_cause_of_injury_code_qualifier import ExternalCauseOfInjuryCodeQualifier
from .health_care_code_information_new import HealthCareCodeInformationNew
from .icd_code import IcdCode
from .present_on_admission_indicator_code import PresentOnAdmissionIndicatorCode


class ExternalCauseOfInjuryNew(HealthCareCodeInformationNew):
    """
    This corresponds to BN or ABN code qualifier values.
    """

    external_cause_of_injury_code_qualifier: ExternalCauseOfInjuryCodeQualifier
    external_cause_of_injury_code: IcdCode
    present_on_admission_indicator: typing.Optional[PresentOnAdmissionIndicatorCode] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        AccountType,
        Allocation,
        AllocationCreate,
        AllocationEarmarkType,
        AllocationEarmarkType_DateOfService,
        AllocationEarmarkType_ExternalEncounterId,
        AllocationTarget,
        AllocationTargetCreate,
        AllocationTargetCreate_AppointmentByIdAndPatientExternalId,
        AllocationTargetCreate_BillingProviderById,
        AllocationTargetCreate_ClaimByEncounterExternalId,
        AllocationTargetCreate_ClaimById,
        AllocationTargetCreate_ServiceLineById,
        AllocationTargetCreate_Unattributed,
        AllocationTarget_Appointment,
        AllocationTarget_BillingProviderId,
        AllocationTarget_Claim,
        AllocationTarget_ServiceLine,
        AllocationTarget_Unattributed,
        AppointmentAllocationTarget,
        AppointmentByIdAndPatientExternalId,
        BalanceEarmark,
        BillingProviderAllocationTarget,
        ClaimAllocationTarget,
        InvoiceUpdate,
        InvoiceUpdate_Remove,
        InvoiceUpdate_Set,
        NoteUpdate,
        NoteUpdate_Remove,
        NoteUpdate_Set,
        PatientTransactionSource,
        RefundReason,
        RefundReasonUpdate,
        RefundReasonUpdate_Remove,
        RefundReasonUpdate_Set,
        ServiceLineAllocationTarget,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "AccountType": ".types",
    "Allocation": ".types",
    "AllocationCreate": ".types",
    "AllocationEarmarkType": ".types",
    "AllocationEarmarkType_DateOfService": ".types",
    "AllocationEarmarkType_ExternalEncounterId": ".types",
    "AllocationTarget": ".types",
    "AllocationTargetCreate": ".types",
    "AllocationTargetCreate_AppointmentByIdAndPatientExternalId": ".types",
    "AllocationTargetCreate_BillingProviderById": ".types",
    "AllocationTargetCreate_ClaimByEncounterExternalId": ".types",
    "AllocationTargetCreate_ClaimById": ".types",
    "AllocationTargetCreate_ServiceLineById": ".types",
    "AllocationTargetCreate_Unattributed": ".types",
    "AllocationTarget_Appointment": ".types",
    "AllocationTarget_BillingProviderId": ".types",
    "AllocationTarget_Claim": ".types",
    "AllocationTarget_ServiceLine": ".types",
    "AllocationTarget_Unattributed": ".types",
    "AppointmentAllocationTarget": ".types",
    "AppointmentByIdAndPatientExternalId": ".types",
    "BalanceEarmark": ".types",
    "BillingProviderAllocationTarget": ".types",
    "ClaimAllocationTarget": ".types",
    "InvoiceUpdate": ".types",
    "InvoiceUpdate_Remove": ".types",
    "InvoiceUpdate_Set": ".types",
    "NoteUpdate": ".types",
    "NoteUpdate_Remove": ".types",
    "NoteUpdate_Set": ".types",
    "PatientTransactionSource": ".types",
    "RefundReason": ".types",
    "RefundReasonUpdate": ".types",
    "RefundReasonUpdate_Remove": ".types",
    "RefundReasonUpdate_Set": ".types",
    "ServiceLineAllocationTarget": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AccountType",
    "Allocation",
    "AllocationCreate",
    "AllocationEarmarkType",
    "AllocationEarmarkType_DateOfService",
    "AllocationEarmarkType_ExternalEncounterId",
    "AllocationTarget",
    "AllocationTargetCreate",
    "AllocationTargetCreate_AppointmentByIdAndPatientExternalId",
    "AllocationTargetCreate_BillingProviderById",
    "AllocationTargetCreate_ClaimByEncounterExternalId",
    "AllocationTargetCreate_ClaimById",
    "AllocationTargetCreate_ServiceLineById",
    "AllocationTargetCreate_Unattributed",
    "AllocationTarget_Appointment",
    "AllocationTarget_BillingProviderId",
    "AllocationTarget_Claim",
    "AllocationTarget_ServiceLine",
    "AllocationTarget_Unattributed",
    "AppointmentAllocationTarget",
    "AppointmentByIdAndPatientExternalId",
    "BalanceEarmark",
    "BillingProviderAllocationTarget",
    "ClaimAllocationTarget",
    "InvoiceUpdate",
    "InvoiceUpdate_Remove",
    "InvoiceUpdate_Set",
    "NoteUpdate",
    "NoteUpdate_Remove",
    "NoteUpdate_Set",
    "PatientTransactionSource",
    "RefundReason",
    "RefundReasonUpdate",
    "RefundReasonUpdate_Remove",
    "RefundReasonUpdate_Set",
    "ServiceLineAllocationTarget",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawEncountersClient, RawEncountersClient

if typing.TYPE_CHECKING:
    from .resources.v_4.client import AsyncV4Client, V4Client


class EncountersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEncountersClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._v_4: typing.Optional[V4Client] = None

    @property
    def with_raw_response(self) -> RawEncountersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEncountersClient
        """
        return self._raw_client

    @property
    def v_4(self):
        if self._v_4 is None:
            from .resources.v_4.client import V4Client  # noqa: E402

            self._v_4 = V4Client(client_wrapper=self._client_wrapper)
        return self._v_4


class AsyncEncountersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEncountersClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._v_4: typing.Optional[AsyncV4Client] = None

    @property
    def with_raw_response(self) -> AsyncRawEncountersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEncountersClient
        """
        return self._raw_client

    @property
    def v_4(self):
        if self._v_4 is None:
            from .resources.v_4.client import AsyncV4Client  # noqa: E402

            self._v_4 = AsyncV4Client(client_wrapper=self._client_wrapper)
        return self._v_4



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .billing_provider_secondary_identification_qualifier import BillingProviderSecondaryIdentificationQualifier


class BillingProviderSecondaryIdentification(UniversalBaseModel):
    reference_identification: str = pydantic.Field()
    """
    Represents REF02 on the EDI 837 Loop 2010BB Billing Provider Secondary Identification segment. Value cannot exceed 50 characters
    """

    reference_identification_qualifier: typing.Optional[BillingProviderSecondaryIdentificationQualifier] = (
        pydantic.Field(default=None)
    )
    """
    Represents REF01 on the EDI 837 Loop 2010BB Billing Provider Secondary Identification segment.
    Valid values include G2 (Provider Commercial Number) and LU (Location Number). Defaults to G2 if not set.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .benefits_related_entity import BenefitsRelatedEntity
from .plan_coverage import PlanCoverage
from .service_coverage import ServiceCoverage


class CoverageBenefits(UniversalBaseModel):
    plan_coverage: typing.Optional[PlanCoverage] = None
    service_specific_coverage: typing.Optional[typing.List[ServiceCoverage]] = None
    benefits_related_entities: typing.Optional[typing.List[BenefitsRelatedEntity]] = None
    notes: typing.Optional[str] = None
    auto_updated_eligibility_check_id: typing.Optional[str] = pydantic.Field(
        alias="autoUpdatedEligibilityCheckId", default=None
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        DimensionMatch,
        DimensionName,
        Dimensions,
        DimensionsPage,
        MatchCptCode,
        MatchDate,
        MatchFacilityTypeCode,
        MatchGeo,
        MatchLicenseType,
        MatchModifiers,
        MatchNetworkTypes,
        MatchPayer,
        MatchPayerPlanGroups,
        MatchProvider,
        MatchResult,
        MatchTestResult,
        NewRate,
        NewRateVersion,
        OptionalDimensions,
        OverlappingRateEntriesError,
        PayerPlanGroupDoesNotMatchRatePayerError,
        PayerThreshold,
        PayerThresholdsPage,
        Rate,
        RateEntry,
        RateUpload,
        RateUploadWithPossibleErrors,
        RateUpload_NewRate,
        RateUpload_NewVersion,
        RatesPage,
        ThresholdMatch,
        ValidationError,
        ValidationError_DuplicateRate,
        ValidationError_EmptyEntries,
        ValidationError_OrganizationProviderNotFound,
        ValidationError_OverlappingRateEntries,
        ValidationError_PayerPlanGroupDoesNotMatchRatePayer,
        ValidationError_PayerPlanGroupNetworkTypeMutualExclusion,
        ValidationError_PayerPlanGroupNotFound,
        ValidationError_VersionConflict,
    )
    from .errors import FailedToBuildServiceLineDimensions, FeeScheduleValidationHttpError
_dynamic_imports: typing.Dict[str, str] = {
    "DimensionMatch": ".types",
    "DimensionName": ".types",
    "Dimensions": ".types",
    "DimensionsPage": ".types",
    "FailedToBuildServiceLineDimensions": ".errors",
    "FeeScheduleValidationHttpError": ".errors",
    "MatchCptCode": ".types",
    "MatchDate": ".types",
    "MatchFacilityTypeCode": ".types",
    "MatchGeo": ".types",
    "MatchLicenseType": ".types",
    "MatchModifiers": ".types",
    "MatchNetworkTypes": ".types",
    "MatchPayer": ".types",
    "MatchPayerPlanGroups": ".types",
    "MatchProvider": ".types",
    "MatchResult": ".types",
    "MatchTestResult": ".types",
    "NewRate": ".types",
    "NewRateVersion": ".types",
    "OptionalDimensions": ".types",
    "OverlappingRateEntriesError": ".types",
    "PayerPlanGroupDoesNotMatchRatePayerError": ".types",
    "PayerThreshold": ".types",
    "PayerThresholdsPage": ".types",
    "Rate": ".types",
    "RateEntry": ".types",
    "RateUpload": ".types",
    "RateUploadWithPossibleErrors": ".types",
    "RateUpload_NewRate": ".types",
    "RateUpload_NewVersion": ".types",
    "RatesPage": ".types",
    "ThresholdMatch": ".types",
    "ValidationError": ".types",
    "ValidationError_DuplicateRate": ".types",
    "ValidationError_EmptyEntries": ".types",
    "ValidationError_OrganizationProviderNotFound": ".types",
    "ValidationError_OverlappingRateEntries": ".types",
    "ValidationError_PayerPlanGroupDoesNotMatchRatePayer": ".types",
    "ValidationError_PayerPlanGroupNetworkTypeMutualExclusion": ".types",
    "ValidationError_PayerPlanGroupNotFound": ".types",
    "ValidationError_VersionConflict": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "DimensionMatch",
    "DimensionName",
    "Dimensions",
    "DimensionsPage",
    "FailedToBuildServiceLineDimensions",
    "FeeScheduleValidationHttpError",
    "MatchCptCode",
    "MatchDate",
    "MatchFacilityTypeCode",
    "MatchGeo",
    "MatchLicenseType",
    "MatchModifiers",
    "MatchNetworkTypes",
    "MatchPayer",
    "MatchPayerPlanGroups",
    "MatchProvider",
    "MatchResult",
    "MatchTestResult",
    "NewRate",
    "NewRateVersion",
    "OptionalDimensions",
    "OverlappingRateEntriesError",
    "PayerPlanGroupDoesNotMatchRatePayerError",
    "PayerThreshold",
    "PayerThresholdsPage",
    "Rate",
    "RateEntry",
    "RateUpload",
    "RateUploadWithPossibleErrors",
    "RateUpload_NewRate",
    "RateUpload_NewVersion",
    "RatesPage",
    "ThresholdMatch",
    "ValidationError",
    "ValidationError_DuplicateRate",
    "ValidationError_EmptyEntries",
    "ValidationError_OrganizationProviderNotFound",
    "ValidationError_OverlappingRateEntries",
    "ValidationError_PayerPlanGroupDoesNotMatchRatePayer",
    "ValidationError_PayerPlanGroupNetworkTypeMutualExclusion",
    "ValidationError_PayerPlanGroupNotFound",
    "ValidationError_VersionConflict",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core import enum

T_Result = typing.TypeVar("T_Result")


class DelayReasonCode(enum.StrEnum):
    """
    Code indicating the reason why a request was delayed
    """

    C_1 = "1"
    """
    Proof of Eligibility Unknown or Unavailable
    """

    C_2 = "2"
    """
    Litigation
    """

    C_3 = "3"
    """
    Authorization Delays
    """

    C_4 = "4"
    """
    Delay in Certifying Provider
    """

    C_5 = "5"
    """
    Delay in Supplying Billing Forms
    """

    C_6 = "6"
    """
    Delay in Delivery of Custom-made Appliances
    """

    C_7 = "7"
    """
    Third Party Processing Delay
    """

    C_8 = "8"
    """
    Delay in Eligibility Determination
    """

    C_9 = "9"
    """
    Original Claim Rejected or Denied Due to a Reason Unrelated to the Billing Limitation Rules
    """

    C_10 = "10"
    """
    Administration Delay in the Prior Approval Process
    """

    C_11 = "11"
    """
    Other
    """

    C_15 = "15"
    """
    Natural Disaster
    """

    C_16 = "16"
    """
    Lack of Information
    """

    C_17 = "17"
    """
    No response to initial request
    """

    _UNKNOWN = "__DELAYREASONCODE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "DelayReasonCode":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        c_1: typing.Callable[[], T_Result],
        c_2: typing.Callable[[], T_Result],
        c_3: typing.Callable[[], T_Result],
        c_4: typing.Callable[[], T_Result],
        c_5: typing.Callable[[], T_Result],
        c_6: typing.Callable[[], T_Result],
        c_7: typing.Callable[[], T_Result],
        c_8: typing.Callable[[], T_Result],
        c_9: typing.Callable[[], T_Result],
        c_10: typing.Callable[[], T_Result],
        c_11: typing.Callable[[], T_Result],
        c_15: typing.Callable[[], T_Result],
        c_16: typing.Callable[[], T_Result],
        c_17: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is DelayReasonCode.C_1:
            return c_1()
        if self is DelayReasonCode.C_2:
            return c_2()
        if self is DelayReasonCode.C_3:
            return c_3()
        if self is DelayReasonCode.C_4:
            return c_4()
        if self is DelayReasonCode.C_5:
            return c_5()
        if self is DelayReasonCode.C_6:
            return c_6()
        if self is DelayReasonCode.C_7:
            return c_7()
        if self is DelayReasonCode.C_8:
            return c_8()
        if self is DelayReasonCode.C_9:
            return c_9()
        if self is DelayReasonCode.C_10:
            return c_10()
        if self is DelayReasonCode.C_11:
            return c_11()
        if self is DelayReasonCode.C_15:
            return c_15()
        if self is DelayReasonCode.C_16:
            return c_16()
        if self is DelayReasonCode.C_17:
            return c_17()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....commons.types.facility_type_code import FacilityTypeCode
from .....diagnoses.types.diagnosis_create import DiagnosisCreate
from .....encounter_providers.resources.v_2.types.rendering_provider import RenderingProvider
from .....encounters_universal.types.universal_encounter_create_from_pre_encounter_base import (
    UniversalEncounterCreateFromPreEncounterBase,
)
from .....service_lines.resources.v_2.types.service_line_create import ServiceLineCreate


class EncounterCreateFromPreEncounter(UniversalEncounterCreateFromPreEncounterBase):
    rendering_provider: RenderingProvider = pydantic.Field()
    """
    The rendering provider is the practitioner -- physician, nurse practitioner, etc. -- performing the service.
    For telehealth services, the rendering provider performs the visit, asynchronous communication, or other service. The rendering provider address should generally be the same as the service facility address.
    """

    place_of_service_code: FacilityTypeCode = pydantic.Field()
    """
    Box 24B on the CMS-1500 claim form. 837p Loop2300, CLM-05-1. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).
    """

    diagnoses: typing.List[DiagnosisCreate] = pydantic.Field()
    """
    Ideally, this field should contain no more than 12 diagnoses. However, more diagnoses
    may be submitted at this time, and coders will later prioritize the 12 that will be
    submitted to the payor.
    """

    service_lines: typing.Optional[typing.List[ServiceLineCreate]] = pydantic.Field(default=None)
    """
    Each service line must be linked to a diagnosis. Concretely,
    `service_line.diagnosis_pointers`must contain at least one entry which should be
    in bounds of the diagnoses list field.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unauthorized_error import UnauthorizedError
from ....commons.errors.unprocessable_entity_error import UnprocessableEntityError
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.payer_plan_group_id import PayerPlanGroupId
from ....commons.types.sort_direction import SortDirection
from ....commons.types.source_of_payment_code import SourceOfPaymentCode
from ....commons.types.unauthorized_error_message import UnauthorizedErrorMessage
from ....commons.types.unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from ....payers.resources.v_3.types.payer_id import PayerId
from ....payers.resources.v_3.types.payer_uuid import PayerUuid
from .errors.payer_plan_group_already_exists_http_error import PayerPlanGroupAlreadyExistsHttpError
from .types.mutable_payer_plan_group import MutablePayerPlanGroup
from .types.payer_plan_group import PayerPlanGroup
from .types.payer_plan_group_already_exists_error import PayerPlanGroupAlreadyExistsError
from .types.payer_plan_group_page import PayerPlanGroupPage
from .types.payer_plan_group_sort_field import PayerPlanGroupSortField

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_multi(
        self,
        *,
        plan_group_name: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        payer_uuid: typing.Optional[typing.Union[PayerUuid, typing.Sequence[PayerUuid]]] = None,
        payer_id: typing.Optional[typing.Union[PayerId, typing.Sequence[PayerId]]] = None,
        plan_type: typing.Optional[typing.Union[SourceOfPaymentCode, typing.Sequence[SourceOfPaymentCode]]] = None,
        is_active: typing.Optional[bool] = None,
        payer_plan_group_id: typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]] = None,
        limit: typing.Optional[int] = None,
        sort_by_similarity: typing.Optional[str] = None,
        sort: typing.Optional[PayerPlanGroupSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayerPlanGroupPage]:
        """
        Returns all payer plan groups matching filter criteria.

        Parameters
        ----------
        plan_group_name : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        payer_uuid : typing.Optional[typing.Union[PayerUuid, typing.Sequence[PayerUuid]]]

        payer_id : typing.Optional[typing.Union[PayerId, typing.Sequence[PayerId]]]

        plan_type : typing.Optional[typing.Union[SourceOfPaymentCode, typing.Sequence[SourceOfPaymentCode]]]

        is_active : typing.Optional[bool]

        payer_plan_group_id : typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]]

        limit : typing.Optional[int]
            Defaults to 100. Cannot exc

        sort_by_similarity : typing.Optional[str]
            If this property is passed, the results will be ordered by those that contain a payer_id, payer_name, plan_group_name, or
            payer_address most similar to the value passed. This will take precedence over the sort and sort_direction properties. This
            will always sort in order of most similar to least similar.

        sort : typing.Optional[PayerPlanGroupSortField]
            Defaults to plan_group_name. If sort_by_similarity is passed, that sort will takes precedence over this property.

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to ascending order if not provided.

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayerPlanGroupPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/payer-plan-groups/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "plan_group_name": plan_group_name,
                "payer_uuid": payer_uuid,
                "payer_id": payer_id,
                "plan_type": plan_type,
                "is_active": is_active,
                "payer_plan_group_id": payer_plan_group_id,
                "limit": limit,
                "sort_by_similarity": sort_by_similarity,
                "sort": sort,
                "sort_direction": sort_direction,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerPlanGroupPage,
                parse_obj_as(
                    type_=PayerPlanGroupPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, payer_plan_group_id: PayerPlanGroupId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayerPlanGroup]:
        """
        Return a plan group with a given ID.

        Parameters
        ----------
        payer_plan_group_id : PayerPlanGroupId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayerPlanGroup]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/payer-plan-groups/v1/{jsonable_encoder(payer_plan_group_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerPlanGroup,
                parse_obj_as(
                    type_=PayerPlanGroup,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, *, request: MutablePayerPlanGroup, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayerPlanGroup]:
        """
        Create a payer plan group

        Parameters
        ----------
        request : MutablePayerPlanGroup

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayerPlanGroup]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/payer-plan-groups/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerPlanGroup,
                parse_obj_as(
                    type_=PayerPlanGroup,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupAlreadyExistsHttpError":
                raise PayerPlanGroupAlreadyExistsHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupAlreadyExistsError,
                        parse_obj_as(
                            type_=PayerPlanGroupAlreadyExistsError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        payer_plan_group_id: PayerPlanGroupId,
        *,
        request: MutablePayerPlanGroup,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayerPlanGroup]:
        """
        Update any of the fields on a payer plan group

        Parameters
        ----------
        payer_plan_group_id : PayerPlanGroupId

        request : MutablePayerPlanGroup

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayerPlanGroup]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/payer-plan-groups/v1/{jsonable_encoder(payer_plan_group_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerPlanGroup,
                parse_obj_as(
                    type_=PayerPlanGroup,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupAlreadyExistsHttpError":
                raise PayerPlanGroupAlreadyExistsHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupAlreadyExistsError,
                        parse_obj_as(
                            type_=PayerPlanGroupAlreadyExistsError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def deactivate(
        self, payer_plan_group_id: PayerPlanGroupId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayerPlanGroup]:
        """
        Marks the payer plan group as deactivated

        Parameters
        ----------
        payer_plan_group_id : PayerPlanGroupId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayerPlanGroup]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/payer-plan-groups/v1/{jsonable_encoder(payer_plan_group_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerPlanGroup,
                parse_obj_as(
                    type_=PayerPlanGroup,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_multi(
        self,
        *,
        plan_group_name: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        payer_uuid: typing.Optional[typing.Union[PayerUuid, typing.Sequence[PayerUuid]]] = None,
        payer_id: typing.Optional[typing.Union[PayerId, typing.Sequence[PayerId]]] = None,
        plan_type: typing.Optional[typing.Union[SourceOfPaymentCode, typing.Sequence[SourceOfPaymentCode]]] = None,
        is_active: typing.Optional[bool] = None,
        payer_plan_group_id: typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]] = None,
        limit: typing.Optional[int] = None,
        sort_by_similarity: typing.Optional[str] = None,
        sort: typing.Optional[PayerPlanGroupSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayerPlanGroupPage]:
        """
        Returns all payer plan groups matching filter criteria.

        Parameters
        ----------
        plan_group_name : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        payer_uuid : typing.Optional[typing.Union[PayerUuid, typing.Sequence[PayerUuid]]]

        payer_id : typing.Optional[typing.Union[PayerId, typing.Sequence[PayerId]]]

        plan_type : typing.Optional[typing.Union[SourceOfPaymentCode, typing.Sequence[SourceOfPaymentCode]]]

        is_active : typing.Optional[bool]

        payer_plan_group_id : typing.Optional[typing.Union[PayerPlanGroupId, typing.Sequence[PayerPlanGroupId]]]

        limit : typing.Optional[int]
            Defaults to 100. Cannot exc

        sort_by_similarity : typing.Optional[str]
            If this property is passed, the results will be ordered by those that contain a payer_id, payer_name, plan_group_name, or
            payer_address most similar to the value passed. This will take precedence over the sort and sort_direction properties. This
            will always sort in order of most similar to least similar.

        sort : typing.Optional[PayerPlanGroupSortField]
            Defaults to plan_group_name. If sort_by_similarity is passed, that sort will takes precedence over this property.

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to ascending order if not provided.

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayerPlanGroupPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/payer-plan-groups/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "plan_group_name": plan_group_name,
                "payer_uuid": payer_uuid,
                "payer_id": payer_id,
                "plan_type": plan_type,
                "is_active": is_active,
                "payer_plan_group_id": payer_plan_group_id,
                "limit": limit,
                "sort_by_similarity": sort_by_similarity,
                "sort": sort,
                "sort_direction": sort_direction,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerPlanGroupPage,
                parse_obj_as(
                    type_=PayerPlanGroupPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, payer_plan_group_id: PayerPlanGroupId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayerPlanGroup]:
        """
        Return a plan group with a given ID.

        Parameters
        ----------
        payer_plan_group_id : PayerPlanGroupId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayerPlanGroup]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/payer-plan-groups/v1/{jsonable_encoder(payer_plan_group_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerPlanGroup,
                parse_obj_as(
                    type_=PayerPlanGroup,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, *, request: MutablePayerPlanGroup, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayerPlanGroup]:
        """
        Create a payer plan group

        Parameters
        ----------
        request : MutablePayerPlanGroup

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayerPlanGroup]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/payer-plan-groups/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerPlanGroup,
                parse_obj_as(
                    type_=PayerPlanGroup,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupAlreadyExistsHttpError":
                raise PayerPlanGroupAlreadyExistsHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupAlreadyExistsError,
                        parse_obj_as(
                            type_=PayerPlanGroupAlreadyExistsError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        payer_plan_group_id: PayerPlanGroupId,
        *,
        request: MutablePayerPlanGroup,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayerPlanGroup]:
        """
        Update any of the fields on a payer plan group

        Parameters
        ----------
        payer_plan_group_id : PayerPlanGroupId

        request : MutablePayerPlanGroup

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayerPlanGroup]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/payer-plan-groups/v1/{jsonable_encoder(payer_plan_group_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerPlanGroup,
                parse_obj_as(
                    type_=PayerPlanGroup,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupAlreadyExistsHttpError":
                raise PayerPlanGroupAlreadyExistsHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupAlreadyExistsError,
                        parse_obj_as(
                            type_=PayerPlanGroupAlreadyExistsError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def deactivate(
        self, payer_plan_group_id: PayerPlanGroupId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayerPlanGroup]:
        """
        Marks the payer plan group as deactivated

        Parameters
        ----------
        payer_plan_group_id : PayerPlanGroupId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayerPlanGroup]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/payer-plan-groups/v1/{jsonable_encoder(payer_plan_group_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerPlanGroup,
                parse_obj_as(
                    type_=PayerPlanGroup,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class ServiceFacilityOperationalStatus(enum.StrEnum):
    """
    Examples
    --------
    from candid.resources.organization_service_facilities.resources.v_2 import (
        ServiceFacilityOperationalStatus,
    )

    ServiceFacilityOperationalStatus.CLOSED
    """

    CLOSED = "C"
    """
    The location is not operational and is closed.
    """

    HOUSEKEEPING = "H"
    """
    The location is currently undergoing cleaning or maintenance.
    """

    ISOLATED = "I"
    """
    The location is quarantined or isolated for special cases.
    """

    CONTAMINATED = "K"
    """
    The location has been identified as contaminated and is not currently usable.
    """

    OCCUPIED = "O"
    """
    The location is currently in use or occupied.
    """

    UNOCCUPIED = "U"
    """
    The location is operational but currently empty or not in use.
    """

    _UNKNOWN = "__SERVICEFACILITYOPERATIONALSTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ServiceFacilityOperationalStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        closed: typing.Callable[[], T_Result],
        housekeeping: typing.Callable[[], T_Result],
        isolated: typing.Callable[[], T_Result],
        contaminated: typing.Callable[[], T_Result],
        occupied: typing.Callable[[], T_Result],
        unoccupied: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ServiceFacilityOperationalStatus.CLOSED:
            return closed()
        if self is ServiceFacilityOperationalStatus.HOUSEKEEPING:
            return housekeeping()
        if self is ServiceFacilityOperationalStatus.ISOLATED:
            return isolated()
        if self is ServiceFacilityOperationalStatus.CONTAMINATED:
            return contaminated()
        if self is ServiceFacilityOperationalStatus.OCCUPIED:
            return occupied()
        if self is ServiceFacilityOperationalStatus.UNOCCUPIED:
            return unoccupied()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.rate_id import RateId
from .rate_entry import RateEntry


class NewRateVersion(UniversalBaseModel):
    rate_id: RateId
    previous_version: int = pydantic.Field()
    """
    New versions of rates must indicate the exact version they modify. When the system attempts to save this new version, if the latest version in the system does not equal this previos_version, the request will be rejected with a EntityConflictError.
    """

    entries: typing.List[RateEntry]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class PatientHistoryCategoryEnum(enum.StrEnum):
    PRESENT_ILLNESS = "present_illness"
    MEDICAL = "medical"
    FAMILY = "family"
    SOCIAL = "social"
    _UNKNOWN = "__PATIENTHISTORYCATEGORYENUM_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "PatientHistoryCategoryEnum":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        present_illness: typing.Callable[[], T_Result],
        medical: typing.Callable[[], T_Result],
        family: typing.Callable[[], T_Result],
        social: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is PatientHistoryCategoryEnum.PRESENT_ILLNESS:
            return present_illness()
        if self is PatientHistoryCategoryEnum.MEDICAL:
            return medical()
        if self is PatientHistoryCategoryEnum.FAMILY:
            return family()
        if self is PatientHistoryCategoryEnum.SOCIAL:
            return social()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....commons.types.page_token import PageToken
from .raw_client import AsyncRawV1Client, RawV1Client
from .types.event import Event
from .types.event_id import EventId
from .types.event_scan_page import EventScanPage


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV1Client
        """
        return self._raw_client

    def scan(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        event_types: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        created_before: typing.Optional[dt.datetime] = None,
        created_after: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EventScanPage:
        """
        Scans the last 30 days of events. All results are sorted by created date, descending.

        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Number of events to return. Minimum value is 1, maximum is 100. Defaults to 10.

        event_types : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Event types to filter on. Defaults to showing all event types.

            | Event Type | Description |
            |------------|-------------|
            | `CandidApi.Encounter.Created` | When a new claim is created |
            | `CandidApi.Encounter.StatusUpdated` | When a claim's status has been updated |
            | `PreEncounter.Patient.Created` | When a new patient has been created |
            | `PreEncounter.Patient.Updated` | When a patient's information has been updated |
            | `PreEncounter.Coverage.Created` | When a new coverage has been created for a patient |
            | `PreEncounter.Coverage.Updated` | When a patient's coverage has been updated |
            | `PreEncounter.Tag.Created` | When a new tag has been created for a patient |
            | `PreEncounter.Tag.Updated` | When a patient's tag has been updated |

        created_before : typing.Optional[dt.datetime]
            Filters for only events created before this time (inclusive).

        created_after : typing.Optional[dt.datetime]
            Filters for only events created after this time (inclusive).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventScanPage

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.events.v_1.scan()
        """
        _response = self._raw_client.scan(
            page_token=page_token,
            limit=limit,
            event_types=event_types,
            created_before=created_before,
            created_after=created_after,
            request_options=request_options,
        )
        return _response.data

    def get(self, event_id: EventId, *, request_options: typing.Optional[RequestOptions] = None) -> Event:
        """
        Parameters
        ----------
        event_id : EventId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Event

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.events.v_1.get(
            event_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get(event_id, request_options=request_options)
        return _response.data


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV1Client
        """
        return self._raw_client

    async def scan(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        event_types: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        created_before: typing.Optional[dt.datetime] = None,
        created_after: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EventScanPage:
        """
        Scans the last 30 days of events. All results are sorted by created date, descending.

        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Number of events to return. Minimum value is 1, maximum is 100. Defaults to 10.

        event_types : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Event types to filter on. Defaults to showing all event types.

            | Event Type | Description |
            |------------|-------------|
            | `CandidApi.Encounter.Created` | When a new claim is created |
            | `CandidApi.Encounter.StatusUpdated` | When a claim's status has been updated |
            | `PreEncounter.Patient.Created` | When a new patient has been created |
            | `PreEncounter.Patient.Updated` | When a patient's information has been updated |
            | `PreEncounter.Coverage.Created` | When a new coverage has been created for a patient |
            | `PreEncounter.Coverage.Updated` | When a patient's coverage has been updated |
            | `PreEncounter.Tag.Created` | When a new tag has been created for a patient |
            | `PreEncounter.Tag.Updated` | When a patient's tag has been updated |

        created_before : typing.Optional[dt.datetime]
            Filters for only events created before this time (inclusive).

        created_after : typing.Optional[dt.datetime]
            Filters for only events created after this time (inclusive).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventScanPage

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.events.v_1.scan()


        asyncio.run(main())
        """
        _response = await self._raw_client.scan(
            page_token=page_token,
            limit=limit,
            event_types=event_types,
            created_before=created_before,
            created_after=created_after,
            request_options=request_options,
        )
        return _response.data

    async def get(self, event_id: EventId, *, request_options: typing.Optional[RequestOptions] = None) -> Event:
        """
        Parameters
        ----------
        event_id : EventId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Event

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.events.v_1.get(
                event_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(event_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .insurance_card import InsuranceCard
    from .insurance_card_base import InsuranceCardBase
    from .insurance_card_create import InsuranceCardCreate
    from .insurance_card_create_optional import InsuranceCardCreateOptional
    from .insurance_card_id import InsuranceCardId
_dynamic_imports: typing.Dict[str, str] = {
    "InsuranceCard": ".insurance_card",
    "InsuranceCardBase": ".insurance_card_base",
    "InsuranceCardCreate": ".insurance_card_create",
    "InsuranceCardCreateOptional": ".insurance_card_create_optional",
    "InsuranceCardId": ".insurance_card_id",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "InsuranceCard",
    "InsuranceCardBase",
    "InsuranceCardCreate",
    "InsuranceCardCreateOptional",
    "InsuranceCardId",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .authorization import Authorization
    from .authorization_unit import AuthorizationUnit
    from .contact import Contact
    from .coverages_for_related_causes import CoveragesForRelatedCauses
    from .do_not_invoice_reason import DoNotInvoiceReason
    from .external_provenance import ExternalProvenance
    from .filing_order import FilingOrder
    from .guarantor import Guarantor
    from .inferred_patient_metadata import InferredPatientMetadata
    from .marital_status import MaritalStatus
    from .mutable_patient import MutablePatient
    from .mutable_patient_with_mrn import MutablePatientWithMrn
    from .origination_detail import OriginationDetail
    from .patient import Patient
    from .patient_coverage_snapshot import PatientCoverageSnapshot
    from .patient_page import PatientPage
    from .patient_sort_field import PatientSortField
    from .referral import Referral
    from .referral_source import ReferralSource
    from .referral_type import ReferralType
    from .referral_unit import ReferralUnit
    from .specialization_category import SpecializationCategory
_dynamic_imports: typing.Dict[str, str] = {
    "Authorization": ".authorization",
    "AuthorizationUnit": ".authorization_unit",
    "Contact": ".contact",
    "CoveragesForRelatedCauses": ".coverages_for_related_causes",
    "DoNotInvoiceReason": ".do_not_invoice_reason",
    "ExternalProvenance": ".external_provenance",
    "FilingOrder": ".filing_order",
    "Guarantor": ".guarantor",
    "InferredPatientMetadata": ".inferred_patient_metadata",
    "MaritalStatus": ".marital_status",
    "MutablePatient": ".mutable_patient",
    "MutablePatientWithMrn": ".mutable_patient_with_mrn",
    "OriginationDetail": ".origination_detail",
    "Patient": ".patient",
    "PatientCoverageSnapshot": ".patient_coverage_snapshot",
    "PatientPage": ".patient_page",
    "PatientSortField": ".patient_sort_field",
    "Referral": ".referral",
    "ReferralSource": ".referral_source",
    "ReferralType": ".referral_type",
    "ReferralUnit": ".referral_unit",
    "SpecializationCategory": ".specialization_category",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Authorization",
    "AuthorizationUnit",
    "Contact",
    "CoveragesForRelatedCauses",
    "DoNotInvoiceReason",
    "ExternalProvenance",
    "FilingOrder",
    "Guarantor",
    "InferredPatientMetadata",
    "MaritalStatus",
    "MutablePatient",
    "MutablePatientWithMrn",
    "OriginationDetail",
    "Patient",
    "PatientCoverageSnapshot",
    "PatientPage",
    "PatientSortField",
    "Referral",
    "ReferralSource",
    "ReferralType",
    "ReferralUnit",
    "SpecializationCategory",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawExportsClient, RawExportsClient

if typing.TYPE_CHECKING:
    from .resources.v_3.client import AsyncV3Client, V3Client


class ExportsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawExportsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._v_3: typing.Optional[V3Client] = None

    @property
    def with_raw_response(self) -> RawExportsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawExportsClient
        """
        return self._raw_client

    @property
    def v_3(self):
        if self._v_3 is None:
            from .resources.v_3.client import V3Client  # noqa: E402

            self._v_3 = V3Client(client_wrapper=self._client_wrapper)
        return self._v_3


class AsyncExportsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawExportsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._v_3: typing.Optional[AsyncV3Client] = None

    @property
    def with_raw_response(self) -> AsyncRawExportsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawExportsClient
        """
        return self._raw_client

    @property
    def v_3(self):
        if self._v_3 is None:
            from .resources.v_3.client import AsyncV3Client  # noqa: E402

            self._v_3 = AsyncV3Client(client_wrapper=self._client_wrapper)
        return self._v_3



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from .street_address_base import StreetAddressBase


class StreetAddressLongZip(StreetAddressBase):
    """
    Examples
    --------
    from candid.resources.commons import State, StreetAddressLongZip

    StreetAddressLongZip(
        address_1="123 Main St",
        address_2="Apt 1",
        city="New York",
        state=State.NY,
        zip_code="10001",
        zip_plus_four_code="1234",
    )
    """

    zip_plus_four_code: str = pydantic.Field()
    """
    4-digit zip add-on code https://en.wikipedia.org/wiki/ZIP_Code#ZIP+4
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
import uuid

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....commons.types.page_token import PageToken
from ....commons.types.sort_direction import SortDirection
from ....commons.types.state import State
from ..v_2.types.contract_sort_field import ContractSortField
from ..v_2.types.contract_status import ContractStatus
from .raw_client import AsyncRawV3Client, RawV3Client
from .types.contract_create_union import ContractCreateUnion
from .types.contract_id import ContractId
from .types.contract_service_facility import ContractServiceFacility
from .types.contract_service_facility_id import ContractServiceFacilityId
from .types.contract_type import ContractType
from .types.contract_update_union import ContractUpdateUnion
from .types.contract_with_providers_union import ContractWithProvidersUnion
from .types.contracting_provider_id import ContractingProviderId
from .types.contracts_page import ContractsPage
from .types.rendering_providerid import RenderingProviderid

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V3Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV3Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV3Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV3Client
        """
        return self._raw_client

    def get(
        self, contract_id: ContractId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ContractWithProvidersUnion:
        """
        Parameters
        ----------
        contract_id : ContractId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContractWithProvidersUnion

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.contracts.v_3.get(
            contract_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get(contract_id, request_options=request_options)
        return _response.data

    def get_multi(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        type: typing.Optional[ContractType] = None,
        contracting_provider_id: typing.Optional[ContractingProviderId] = None,
        rendering_provider_ids: typing.Optional[
            typing.Union[RenderingProviderid, typing.Sequence[RenderingProviderid]]
        ] = None,
        payer_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        states: typing.Optional[typing.Union[State, typing.Sequence[State]]] = None,
        contract_status: typing.Optional[ContractStatus] = None,
        sort: typing.Optional[ContractSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContractsPage:
        """
        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Max number of contracts returned. Defaults to 1000. Max is 1000.

        type : typing.Optional[ContractType]
            The type of contract

        contracting_provider_id : typing.Optional[ContractingProviderId]

        rendering_provider_ids : typing.Optional[typing.Union[RenderingProviderid, typing.Sequence[RenderingProviderid]]]

        payer_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter to contracts that include any of the included payer names.

        states : typing.Optional[typing.Union[State, typing.Sequence[State]]]

        contract_status : typing.Optional[ContractStatus]
            The status of the contract. Defaults to `pending`

        sort : typing.Optional[ContractSortField]
            Potentially sort by a contract related attribute.  Defaults to created_at

        sort_direction : typing.Optional[SortDirection]
            Direction of sort, defaulting to desc

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContractsPage

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.contracts.v_3.get_multi()
        """
        _response = self._raw_client.get_multi(
            page_token=page_token,
            limit=limit,
            type=type,
            contracting_provider_id=contracting_provider_id,
            rendering_provider_ids=rendering_provider_ids,
            payer_names=payer_names,
            states=states,
            contract_status=contract_status,
            sort=sort,
            sort_direction=sort_direction,
            request_options=request_options,
        )
        return _response.data

    def create(
        self, *, request: ContractCreateUnion, request_options: typing.Optional[RequestOptions] = None
    ) -> ContractWithProvidersUnion:
        """
        Creates a new contract within the user's current organization

        Parameters
        ----------
        request : ContractCreateUnion

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContractWithProvidersUnion

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.commons import Regions_States, State
        from candid.resources.contracts.resources.v_2 import InsuranceTypes
        from candid.resources.contracts.resources.v_3 import (
            ContractCreateUnion_Professional,
            ContractType,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.contracts.v_3.create(
            request=ContractCreateUnion_Professional(
                rendering_provider_ids=[
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    )
                ],
                contract_type=ContractType.PROFESSIONAL,
                contracting_provider_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                payer_uuid=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                effective_date="effective_date",
                regions=Regions_States(
                    states=[State.AA, State.AA],
                ),
                commercial_insurance_types=InsuranceTypes(),
                medicare_insurance_types=InsuranceTypes(),
                medicaid_insurance_types=InsuranceTypes(),
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def delete(self, contract_id: ContractId, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        contract_id : ContractId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.contracts.v_3.delete(
            contract_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.delete(contract_id, request_options=request_options)
        return _response.data

    def update(
        self,
        contract_id: ContractId,
        *,
        request: ContractUpdateUnion,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContractWithProvidersUnion:
        """
        Parameters
        ----------
        contract_id : ContractId

        request : ContractUpdateUnion

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContractWithProvidersUnion

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.contracts.resources.v_3 import (
            ContractUpdateUnion_Professional,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.contracts.v_3.update(
            contract_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=ContractUpdateUnion_Professional(),
        )
        """
        _response = self._raw_client.update(contract_id, request=request, request_options=request_options)
        return _response.data

    def create_contract_service_facility(
        self,
        contract_id: ContractId,
        *,
        service_facility_id: uuid.UUID,
        provider_ids: typing.Set[uuid.UUID],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContractServiceFacility:
        """
        Parameters
        ----------
        contract_id : ContractId

        service_facility_id : uuid.UUID
            The UUID of the service facility

        provider_ids : typing.Set[uuid.UUID]
            The providers who are authorized under the contract

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContractServiceFacility

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.contracts.v_3.create_contract_service_facility(
            contract_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            service_facility_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            provider_ids={
                uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                )
            },
        )
        """
        _response = self._raw_client.create_contract_service_facility(
            contract_id,
            service_facility_id=service_facility_id,
            provider_ids=provider_ids,
            request_options=request_options,
        )
        return _response.data

    def update_contract_service_facility(
        self,
        contract_id: ContractId,
        contract_service_facility_id: ContractServiceFacilityId,
        *,
        service_facility_id: typing.Optional[uuid.UUID] = OMIT,
        provider_ids: typing.Optional[typing.Set[uuid.UUID]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContractServiceFacility:
        """
        Parameters
        ----------
        contract_id : ContractId

        contract_service_facility_id : ContractServiceFacilityId

        service_facility_id : typing.Optional[uuid.UUID]
            The UUID of the service facility

        provider_ids : typing.Optional[typing.Set[uuid.UUID]]
            The providers who are authorized under the contract

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContractServiceFacility

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.contracts.v_3.update_contract_service_facility(
            contract_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            contract_service_facility_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.update_contract_service_facility(
            contract_id,
            contract_service_facility_id,
            service_facility_id=service_facility_id,
            provider_ids=provider_ids,
            request_options=request_options,
        )
        return _response.data


class AsyncV3Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV3Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV3Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV3Client
        """
        return self._raw_client

    async def get(
        self, contract_id: ContractId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ContractWithProvidersUnion:
        """
        Parameters
        ----------
        contract_id : ContractId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContractWithProvidersUnion

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.contracts.v_3.get(
                contract_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(contract_id, request_options=request_options)
        return _response.data

    async def get_multi(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        type: typing.Optional[ContractType] = None,
        contracting_provider_id: typing.Optional[ContractingProviderId] = None,
        rendering_provider_ids: typing.Optional[
            typing.Union[RenderingProviderid, typing.Sequence[RenderingProviderid]]
        ] = None,
        payer_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        states: typing.Optional[typing.Union[State, typing.Sequence[State]]] = None,
        contract_status: typing.Optional[ContractStatus] = None,
        sort: typing.Optional[ContractSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContractsPage:
        """
        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Max number of contracts returned. Defaults to 1000. Max is 1000.

        type : typing.Optional[ContractType]
            The type of contract

        contracting_provider_id : typing.Optional[ContractingProviderId]

        rendering_provider_ids : typing.Optional[typing.Union[RenderingProviderid, typing.Sequence[RenderingProviderid]]]

        payer_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter to contracts that include any of the included payer names.

        states : typing.Optional[typing.Union[State, typing.Sequence[State]]]

        contract_status : typing.Optional[ContractStatus]
            The status of the contract. Defaults to `pending`

        sort : typing.Optional[ContractSortField]
            Potentially sort by a contract related attribute.  Defaults to created_at

        sort_direction : typing.Optional[SortDirection]
            Direction of sort, defaulting to desc

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContractsPage

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.contracts.v_3.get_multi()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_multi(
            page_token=page_token,
            limit=limit,
            type=type,
            contracting_provider_id=contracting_provider_id,
            rendering_provider_ids=rendering_provider_ids,
            payer_names=payer_names,
            states=states,
            contract_status=contract_status,
            sort=sort,
            sort_direction=sort_direction,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self, *, request: ContractCreateUnion, request_options: typing.Optional[RequestOptions] = None
    ) -> ContractWithProvidersUnion:
        """
        Creates a new contract within the user's current organization

        Parameters
        ----------
        request : ContractCreateUnion

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContractWithProvidersUnion

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.commons import Regions_States, State
        from candid.resources.contracts.resources.v_2 import InsuranceTypes
        from candid.resources.contracts.resources.v_3 import (
            ContractCreateUnion_Professional,
            ContractType,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.contracts.v_3.create(
                request=ContractCreateUnion_Professional(
                    rendering_provider_ids=[
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        )
                    ],
                    contract_type=ContractType.PROFESSIONAL,
                    contracting_provider_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    payer_uuid=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    effective_date="effective_date",
                    regions=Regions_States(
                        states=[State.AA, State.AA],
                    ),
                    commercial_insurance_types=InsuranceTypes(),
                    medicare_insurance_types=InsuranceTypes(),
                    medicaid_insurance_types=InsuranceTypes(),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def delete(self, contract_id: ContractId, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        contract_id : ContractId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.contracts.v_3.delete(
                contract_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(contract_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        contract_id: ContractId,
        *,
        request: ContractUpdateUnion,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContractWithProvidersUnion:
        """
        Parameters
        ----------
        contract_id : ContractId

        request : ContractUpdateUnion

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContractWithProvidersUnion

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.contracts.resources.v_3 import (
            ContractUpdateUnion_Professional,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.contracts.v_3.update(
                contract_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=ContractUpdateUnion_Professional(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(contract_id, request=request, request_options=request_options)
        return _response.data

    async def create_contract_service_facility(
        self,
        contract_id: ContractId,
        *,
        service_facility_id: uuid.UUID,
        provider_ids: typing.Set[uuid.UUID],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContractServiceFacility:
        """
        Parameters
        ----------
        contract_id : ContractId

        service_facility_id : uuid.UUID
            The UUID of the service facility

        provider_ids : typing.Set[uuid.UUID]
            The providers who are authorized under the contract

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContractServiceFacility

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.contracts.v_3.create_contract_service_facility(
                contract_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                service_facility_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                provider_ids={
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    )
                },
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_contract_service_facility(
            contract_id,
            service_facility_id=service_facility_id,
            provider_ids=provider_ids,
            request_options=request_options,
        )
        return _response.data

    async def update_contract_service_facility(
        self,
        contract_id: ContractId,
        contract_service_facility_id: ContractServiceFacilityId,
        *,
        service_facility_id: typing.Optional[uuid.UUID] = OMIT,
        provider_ids: typing.Optional[typing.Set[uuid.UUID]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContractServiceFacility:
        """
        Parameters
        ----------
        contract_id : ContractId

        contract_service_facility_id : ContractServiceFacilityId

        service_facility_id : typing.Optional[uuid.UUID]
            The UUID of the service facility

        provider_ids : typing.Optional[typing.Set[uuid.UUID]]
            The providers who are authorized under the contract

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContractServiceFacility

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.contracts.v_3.update_contract_service_facility(
                contract_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                contract_service_facility_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_contract_service_facility(
            contract_id,
            contract_service_facility_id,
            service_facility_id=service_facility_id,
            provider_ids=provider_ids,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.external_provider import ExternalProvider
from .referral_source import ReferralSource
from .referral_type import ReferralType
from .specialization_category import SpecializationCategory


class OriginationDetail(UniversalBaseModel):
    referral_source: ReferralSource
    referring_provider: typing.Optional[ExternalProvider] = None
    specialization_categories: typing.Optional[typing.List[SpecializationCategory]] = None
    referral_type: typing.Optional[ReferralType] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ...commons.types.street_address_short_zip import StreetAddressShortZip
from .individual_base import IndividualBase


class PatientBase(IndividualBase):
    external_id: str = pydantic.Field()
    """
    The ID used to identify this individual in your system. For example, your internal patient ID or an EHR patient ID.
    """

    date_of_birth: dt.date = pydantic.Field()
    """
    Box 3 on the CMS-1500 claim form or Form Locator 10 on a UB-04 claim form. The date format should be in ISO 8601 date; formatted YYYY-MM-DD (i.e. 2012-02-01)
    """

    address: StreetAddressShortZip = pydantic.Field()
    """
    Box 5 on the CMS-1500 claim form or Form Locator 9 on a UB-04 claim form.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class AddressUse(enum.StrEnum):
    HOME = "HOME"
    WORK = "WORK"
    TEMP = "TEMP"
    OLD = "OLD"
    BILLING = "BILLING"
    _UNKNOWN = "__ADDRESSUSE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "AddressUse":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        home: typing.Callable[[], T_Result],
        work: typing.Callable[[], T_Result],
        temp: typing.Callable[[], T_Result],
        old: typing.Callable[[], T_Result],
        billing: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is AddressUse.HOME:
            return home()
        if self is AddressUse.WORK:
            return work()
        if self is AddressUse.TEMP:
            return temp()
        if self is AddressUse.OLD:
            return old()
        if self is AddressUse.BILLING:
            return billing()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .allocation_earmark_type import AllocationEarmarkType
from .allocation_target_create import AllocationTargetCreate


class AllocationCreate(UniversalBaseModel):
    """
    Allocations are portions of payments that are applied to specific resources, known as targets. Each allocation has
    and amount, defined in cents, and a target.
    """

    amount_cents: int
    target: AllocationTargetCreate
    earmark: typing.Optional[AllocationEarmarkType] = pydantic.Field(default=None)
    """
    If enabled for your organization, optional earmarking configuration for patient prepayments.
    When provided on unattributed allocations, holds the payment for future auto-allocation to matching encounters.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .......core.api_error import ApiError
from .......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .......core.http_response import AsyncHttpResponse, HttpResponse
from .......core.jsonable_encoder import jsonable_encoder
from .......core.pydantic_utilities import parse_obj_as
from .......core.request_options import RequestOptions
from ....common.types.page_token import PageToken
from ....common.types.tag_id import TagId
from .types.mutable_tag import MutableTag
from .types.tag import Tag
from .types.tag_page import TagPage

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(self, id: TagId, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Tag]:
        """
        Gets a tag by TagId.

        Parameters
        ----------
        id : TagId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Tag]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tags/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Tag,
                parse_obj_as(
                    type_=Tag,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TagPage]:
        """
        Gets all tags. Defaults to page size of 1000.

        Parameters
        ----------
        limit : typing.Optional[int]

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TagPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "tags/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "limit": limit,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                TagPage,
                parse_obj_as(
                    type_=TagPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, *, request: MutableTag, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Tag]:
        """
        Adds a new tag if it does not already exist, otherwise, returns the existing tag.

        Parameters
        ----------
        request : MutableTag

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Tag]
        """
        _response = self._client_wrapper.httpx_client.request(
            "tags/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Tag,
                parse_obj_as(
                    type_=Tag,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self, id: TagId, version: str, *, request: MutableTag, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Tag]:
        """
        Updates a tag. The path must contain the most recent version to prevent races.

        Parameters
        ----------
        id : TagId

        version : str

        request : MutableTag

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Tag]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tags/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Tag,
                parse_obj_as(
                    type_=Tag,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def deactivate(
        self, id: TagId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Sets a tag as deactivated.  The path must contain the most recent version to prevent races.

        Parameters
        ----------
        id : TagId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tags/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, id: TagId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Tag]:
        """
        Gets a tag by TagId.

        Parameters
        ----------
        id : TagId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Tag]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tags/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Tag,
                parse_obj_as(
                    type_=Tag,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TagPage]:
        """
        Gets all tags. Defaults to page size of 1000.

        Parameters
        ----------
        limit : typing.Optional[int]

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TagPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tags/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "limit": limit,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                TagPage,
                parse_obj_as(
                    type_=TagPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, *, request: MutableTag, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Tag]:
        """
        Adds a new tag if it does not already exist, otherwise, returns the existing tag.

        Parameters
        ----------
        request : MutableTag

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Tag]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tags/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Tag,
                parse_obj_as(
                    type_=Tag,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self, id: TagId, version: str, *, request: MutableTag, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Tag]:
        """
        Updates a tag. The path must contain the most recent version to prevent races.

        Parameters
        ----------
        id : TagId

        version : str

        request : MutableTag

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Tag]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tags/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Tag,
                parse_obj_as(
                    type_=Tag,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def deactivate(
        self, id: TagId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Sets a tag as deactivated.  The path must contain the most recent version to prevent races.

        Parameters
        ----------
        id : TagId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tags/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...commons.types.street_address_short_zip_optional import StreetAddressShortZipOptional


class EncounterServiceFacilityUpdateWithOptionalAddress(UniversalBaseModel):
    organization_name: typing.Optional[str] = None
    npi: typing.Optional[str] = pydantic.Field(default=None)
    """
    An NPI specific to the service facility if applicable, i.e. if it has one and is not under the billing provider's NPI.
    Box 32 section (a) of the CMS-1500 claim form.
    """

    address: typing.Optional[StreetAddressShortZipOptional] = pydantic.Field(default=None)
    """
    zip_plus_four_code is required for service facility address. When the zip_plus_four_code is not available use "9998" as per CMS documentation.
    """

    secondary_identification: typing.Optional[str] = pydantic.Field(default=None)
    """
    An additional identifier for the service facility other than the facility's NPI. Some payers may require this field.
    Potential examples: state license number, provider commercial number, or location number.
    Box 32 section (b) of the CMS-1500 claim form.
    """

    mammography_certification_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The associated mammography certification number for this service facility. This is a 6 digit code assigned by the FDA.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .intake_question_optional import IntakeQuestionOptional
from .patient_history_category_enum import PatientHistoryCategoryEnum


class PatientHistoryCategoryOptional(UniversalBaseModel):
    category: typing.Optional[PatientHistoryCategoryEnum] = None
    questions: typing.Optional[typing.List[IntakeQuestionOptional]] = pydantic.Field(default=None)
    """
    Must contain at least one item.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.additional_payer_information import AdditionalPayerInformation
from .....common.types.payer_id import PayerId
from .....common.types.period import Period
from .authorization_unit import AuthorizationUnit


class Authorization(UniversalBaseModel):
    payer_id: PayerId
    payer_name: str
    additional_payer_information: typing.Optional[AdditionalPayerInformation] = None
    authorization_number: str
    cpt_code: str
    apply_for_all_cpt_codes: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, then the authorization will apply for all claims for the payer that fall in range the `period`.
    """

    no_prior_authorization_required: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, indicates that prior authorization is not required and prior authorization number will not be set on the claim for this authorization.
    """

    units: AuthorizationUnit
    quantity: typing.Optional[int] = None
    period: typing.Optional[Period] = None
    notes: typing.Optional[str] = None
    billing_provider_npi: typing.Optional[str] = pydantic.Field(default=None)
    """
    The NPI of the billing provider for which this authorization applies.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
